#include <cstdlib>
#include <cstdio>
#include <cassert>
#include <iostream>
#include <fstream>

#define MAX_N 256
#define PERIOD 54

#define MAX_CHAR 0x7f

#define pow2(n)  (1<<(n))
#define bit(i,n) (((i)>>(n))&1)

int n;
char wastage[pow2(9)];
bool solution[MAX_N+2][MAX_N+2];
bool periodic[PERIOD];

void calcWastage() {
	for (int i = 0; i < pow2(9); i++) {
		wastage[i] = MAX_CHAR;

		int s = bit(i,8) + bit(i,5) + bit(i,2) + bit(i,7) + bit(i,1) + bit(i,6) + bit(i,3) + bit(i,0);
		if (bit(i,4) && s != 2 && s != 3) continue;
		if (!bit(i,4) && s == 3) continue;

		s = bit(i,7) + bit(i,5) + bit(i,3) + bit(i,1);
		if (bit(i,4)) {
			if (bit(i,8) && bit(i,6)) s++;
			if (bit(i,8) && bit(i,2)) s++;
			if (bit(i,6) && bit(i,0)) s++;
			if (bit(i,2) && bit(i,0)) s++;
		}
		if (s == 4) wastage[i] = 2;
		else if (s < 2) wastage[i] = 2 - s;
		else wastage[i] = 0;
	}
}

bool readSolution() {
	char filename[100];
	char line[MAX_N];
	std::ifstream in_file;

	sprintf(filename, "%d.txt", n);
	in_file.open(filename, std::ios::in);
	if (!in_file.is_open()) {
		if (!periodic[n%PERIOD]) {
			printf("n = %d, solution missing!\n", n);
		} else {
			printf("n = %d, solution can be constructed\n", n);
		}
		return false;
	}

	for (int i = 0; i < n+2; i++) {
		for (int j = 0; j < n+2; j++) {
			solution[i][j] = 0;
		}
	}

	for (int i = 1; i <= n; i++) {
		in_file.getline(line, n+1);
		for (int j = 1; j <= n; j++) solution[i][j] = (line[j-1] == '1');
	}
	if (in_file.fail()) { printf("Solution read failed!\n"); exit(1);}

	return true;
}

void verifySolution() {
	for (int i = 1; i <= n; i++) {
		for (int j = 1; j <= n; j++) {
			int s = 256*solution[i-1][j-1] + 128*solution[i-1][j] + 64*solution[i-1][j+1] + 32*solution[i][j-1] + 16*solution[i][j] + 8*solution[i][j+1] + 4*solution[i+1][j-1] + 2*solution[i+1][j] + solution[i+1][j+1];
			if (wastage[s] == MAX_CHAR) printf("violation: %d %d\n", i, j);
//			assert(wastage[s] != MAX_CHAR);
		}
	}

	for (int i = 2; i <= n-1; i++) {
		if (solution[1][i-1] + solution[1][i] + solution[1][i+1] > 2) {
			printf("violation: %d %d\n", 1, i);
//			assert(false);
		}
		if (solution[i-1][n] + solution[i][n] + solution[i+1][n] > 2) {
			printf("violation: %d %d\n", i, n);
//			assert(false);
		}
		if (solution[n][n+2-i] + solution[n][n+1-i] + solution[n][n-i] > 2) {
			printf("violation: %d %d\n", n, n+1-i);
//			assert(false);
		}
		if (solution[n+2-i][1] + solution[n+1-i][1] + solution[n-i][1] > 2) {
			printf("violation: %d %d\n", n+1-i, 1);
//			assert(false);
		}
	}

	int count = 0;
	for (int i = 1; i <= n; i++) {
		for (int j = 1; j <= n; j++) {
			count += solution[i][j];
		}
	}
	printf("n = %d, live = %d\n", n, count);
}


void findPeriodic() {
	int x = -1, y = -1;

	for (int i = 1; i <= n-3-PERIOD; i++) {
		bool ok = true;
		for (int j = 1; j <= n; j++) {
			if (solution[i][j] != solution[i+PERIOD][j]) ok = false;
			if (solution[i+1][j] != solution[i+PERIOD+1][j]) ok = false;
			if (solution[i+2][j] != solution[i+PERIOD+2][j]) ok = false;
			if (solution[i+3][j] != solution[i+PERIOD+3][j]) ok = false;
			if (!ok) break;
		}
		if (ok) {
			x = i;
			break;
		}
	}

	for (int i = 1; i <= n-3-PERIOD; i++) {
		bool ok = true;
		for (int j = 1; j <= n; j++) {
			if (solution[j][i] != solution[j][i+PERIOD]) ok = false;
			if (solution[j][i+1] != solution[j][i+PERIOD+1]) ok = false;
			if (solution[j][i+2] != solution[j][i+PERIOD+2]) ok = false;
			if (solution[j][i+3] != solution[j][i+PERIOD+3]) ok = false;
			if (!ok) break;
		}
		if (ok) {
			y = i;
			break;
		}
	}

//	if (x != -1) printf("x splice = %d\n", x);
//	if (y != -1) printf("y splice = %d\n", y);

	if (x != -1 && y != -1) periodic[n%PERIOD] = true;
}

int main(int argc, char* argv[]) {
	calcWastage();

	for (n = 20; n <= MAX_N; n++) {
		if (!readSolution()) continue;
		verifySolution();
		findPeriodic();
	}

	printf("Periodics: ");
	for (int i = 0; i < PERIOD; i++) {
		printf("%d ", periodic[i]);
	}
	printf("\n");

	return 0;
}

